/*
 *  (C) Copyright ECWMF http://wwww.ecmwf.int
 *
 *  This software is licensed under the terms of the Apache Licence Version 2.0
 *  which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *  In applying this licence, ECMWF does not waive the privileges and immunities
 *  granted to it by virtue of its status as an intergovernmental organisation nor
 *  does it submit to any jurisdiction.
 *
 */
define(['config', './parameters', 'ajax_queue', 'time', 'utils'], function(ECMWF, Parameters, ajaxqueue, Time, Utils) {

	function BaseProduct() {

		this.data = {product: {}};

		this.set_parameter = function(parameter, value, done) {
			done = done || Utils.noop;
			var self = this;

			var param = typeof parameter === 'object' ? parameter : self.variables[parameter];
			console.assert(param, 'parameter=' + parameter);
			if(!param) return;

			if(!param.is_current(value)){
				self._loaded = false;
				param.set(value);
			} else {
				done();
			}
		}

		this.has_widget = function (widget) {
			return !!~(this.data.product.widgets || []).indexOf(widget)
		}

		this.is_eccharts = function() {
			return false
		}

		/*
		 *  Deprecated - use the simplified availability2 in new code
		 */
		this.availability = function(time, ready) {
			var self = this;

			if (this._availability) {
				self._send_availability(time, ready);
				return;
			}

			ajaxqueue({
				url: '/backend/get-availability/',
				json: {
					product: this.as_json(null, null)
				},
				logErrors: 'Product availability',
				success: function(result) {
					self._update_number = ++BaseProduct.prototype._update_number;
					self._updated = result.updated;
					self._availability = result.availability.map(function(a) {
						return new Time(a);
					})
					self._send_availability(time, ready);
				}
			})
		}

		/*
		 *  Creates an object with the following properties:
		 *
		 *  - current         Time object
		 *  - base_time       object containing all the times for the current valid time?
		 *  - valid_time      object containing all the times for the current base time?
		 *  - availability    array of type Time
		 *  - update_number   integer
		 *  - updated         float   eg 1532959244.242828
		 *  - best_base_time  boolean
		 *  - old_cycle       boolean
		 *
		 *  The values are only valid for the current Time
		 *
		 *  It is recomended to instead get the information from
		 *    Parameters/Time.valid_times()
		 *    Parameters/Time.base_times()
		 */
		this._send_availability = function(time, ready) {
			if (!time) return ready();

			var self = this,
				times = [],
				cycles = [],
				old = false;

			self._availability.forEach(function(d, i) {
				if ((time.base_time.getTime() - d.base_time.getTime()) === 0) {
					times.push(d);
				}

				if (time.base_time.getTime() - d.base_time.getTime() < 0) {
					old = true;
				}

				if ((time.valid_time.getTime() - d.valid_time.getTime()) === 0) {
					cycles.push(d);
				}
			})

			var valid_times = times.sort(function(a, b) {
				return a.valid_time.getTime() - b.valid_time.getTime();
			})

			var base_times = cycles.sort(function(a, b) {
				return a.base_time.getTime() - b.base_time.getTime();
			})

			var a = {
				current: time,
				availability: self._availability,
				valid_time: {
					values: valid_times,
					index: time.index(valid_times)
				},
				base_time: {
					values: base_times,
					index: time.index(base_times)
				},
				update_number: self._update_number,
				updated: self._updated
			}


			// True if there is no better base time for this valid time
			a.best_base_time = (a.base_time.index === a.base_time.values.length - 1);

			// This is an old cycle
			a.old_cycle = old;

			//console.log(a.base_time.values.map(function(a) { return a.key; }));
			//console.log(a.valid_time.values.map(function(a) { return a.key; }));
			ready(a);
		}

		this.print = function(request) {
			ajaxqueue(Utils.extend(
				{
					url: ECMWF.options.ajax.backend.print_chart,
					priority: 'very-high',
					logErrors: 'Print'
				},
				request
			))
		}

		this.add_to_dashboard = function(options, done) {
			var self = this;

			var state = self.get_save_state();

			if(self.has_widget('interactive')) state.state.chart_type = 'interactive';
			ajaxqueue({
				priority: 'very-high',
				url: '/backend/save-product-to-dashboard/',
				json: {
					product: state,
					options: Utils.extend(
						{original_path: (self.has_widget('interactive') || self.has_widget('basic')) ? window.location.pathname : undefined},
						options
					)
				},
				logErrors: 'Save dashboard product',
				success: function(result) {
					done && done(result);
				}
			});
		}

		this.download_url = function(token, email) {
			var self = this
			// TODO: Get this from settings, or use ECMWF.fix_host.to_backend
			var full_host = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port: '');
			var host = full_host.replace(/http:/, 'https:').replace(/www|eccharts/gi, 'apps').replace(/stage|eccharts-test/gi, 'apps-test');
			var name = typeof this.name === 'function' ? this.name() : this.name

			var url = host
				+ '/plots/product-download/' + (this.data.product.package || 'catalogue') + '/' + name + '/'
				+ '?' + Object.keys(this.variables).map(function(k) { var v = self.variables[k]; return v.name + '=' + v.to_string(); }).join('&');

			return encodeURI(url + '&token=' + 'TOKEN' + '&email=' + email);
		}
	}

	BaseProduct.prototype = {
		_update_number: 0,
		_req_defaults: {},

		/*
		 *  This is a low-level method used to set Time.availability
		 *
		 *  To get availability data, users would normally subsribe to a Time
		 *  observable where available.
		 */
		availability2: function(done) {
			ajaxqueue({
				url: '/backend/get-availability/',
				json: {
					product: this.as_json(null, null)
				},
				logErrors: 'Product availability',
				success: function(result) {
					done(result.availability)
				}
			})
		}
	}

	return BaseProduct;
});
