import h from 'virtual-dom/h'
import Button from 'grid/button'
import mixins from 'mixins'
import Utils from 'utils'
import Handler from './handlers/handler'
import './index.css'

var handlers = {
	text: {
		render: function(item) {
			return h('.cell.right', h('textarea', {rows: 3, cols: 50}, item._item.value))
		},
		collect: function(item, row) {
			return row.querySelector('textarea').value
		}
	},
	choice: {
		render: function(item) {
			// TODO menu does not open
			// TODO use a menu widget instead? ie the bootstrap one?
			return h('.cell.right', [h('select', {}, item._item.values.map(function(v){ return h('option', {value: v}, item._item.labels[v]) }))])
		},
		collect: function(item, row) {
			return row.querySelector('select').value
		}
	},
	boolean: {
		render: function(item) {
			return h('.cell.right', [h('input', {type: 'checkbox', checked: item.value() ? 'checked' : ''})])
		},
		collect: function(item, row) {
			return row.querySelector('input').checked
		}
	}
}

Object.keys(handlers).forEach(function(k) {
	handlers[k] = Utils.extend(new Handler(), handlers[k])
})

function Form(args) {

	var o = this.options = Utils.extend({
		el: null, // query to find the form element
		ok: null, // callback function
		show_ok: true,
		ok_label: 'Ok'
	}, args)

	this.items = o.items.map(function(v, i){
		return new FormItem(v, o.values && o.values[v.name] || v.values, o.el, i, o.renderer)
	})
}

Form.prototype = Utils.extend({
	render: function() {
		var buttons = []
		if (this.options.show_ok) {
			buttons.push(new Button({
				text: this.options.ok_label,
				show_text: true,
				on_click: this.bind(this.submit)
			}).render())
		}
		var button_row = h('.buttons.row', buttons)

		var items = this.items.map(function(item){
			return h('.row', [
				h('.cell.left', item._item.title),
				item._handler.render(item)
			])
		})
		items.push(button_row)

		// TODO change the visible Ok button to be a real button
		items.push(h('button', {type: 'submit', style: {display: 'none'}}))

		return h('form.form', {method: 'post', action: '#', onsubmit: function(e) {
			e.preventDefault()
		}}, items)
	},

	collect: function() {
		var result = {};
		var ok = true;

		this.items.forEach(function(item) {
			var val = item.collect();
			result[item.name()] = val;
			item.error('');
			if (!item.check(val)) { ok = false; }
		})

		return {result: result, ok: ok}
	},

	submit: function(){
		document.activeElement.blur()
		document.querySelector(this.options.el + ' form button').click() // Submit the form properly to force change events to be triggered (not always? the blur() above does the same thing anyway)

		var c = this.collect();
		if (c.ok) {
			this.options.ok(c.result);
			return c.result;
		}
	}
}, {bind: mixins.bind})

Form.add_handler = function(name, handler) {
	handlers[name] = handler;
}

function FormItem(item, value, el, row, renderer) {
	this._item = item
	this._value = value
	this._el = el  // query to find the element
	this.row = row // index
	this.renderer = renderer

	// TODO some handlers have state and so need to be instantiated, but others do not
	//      We need a way of indicating if instantiation is needed or not
	//      Do they _really_ need state? state can be stored in the item.
	if(!item.handler){
		if(item.type === 'point')
			item.handler = new handlers.point({renderer: renderer})
	}

	this._handler = item.handler || handlers[item.type];
	if(!this._handler) throw new Error('no form handler for type ' + item.type)
}

FormItem.prototype = {
	values: function() {
		return this._item.values
	},

	name: function() {
		return this._item.name
	},

	value: function() {
		var item = this._item
		return ((item.type === 'float' && typeof this._value !== 'undefined') || this._value) ? this._value : item.value;
	},

	defaults: function () {
		var s = this.value;
		if(s === undefined || s === null) { s = this._item['default'];  }

		while (typeof(s) === 'function') { s = s.apply(this); }

		if(s === undefined || s === null) { s = [];  }

		if (typeof(s) === 'object') { return s; }

		return [s];
	},

	label: function (name, i) {
		var s = this._item.labels, n = name;

		while (typeof(s) === 'function') s = s(name);
		if (typeof(s) === 'string') return s;

		if (Array.isArray(s)) return this._item.labels[i];

		if (typeof(s) === 'object') return s[name];

		if (name === undefined || name === null) {
			name = n;
			if (name === undefined || name === null) {
				name = "?";
			}
		}

		return name;
	},

	collect: function() {
		var el = document.querySelectorAll(this._el + ' .row')
		if(el){
			el = el[this.row]
			return this._handler.collect(this, el);
		}
	},

	check: function(value) {
		return this._handler.check(this, value);
	},

	multiple: function() {
		return this._item.multiple
	},

	error: function(msg) {
		//this.error_text.html(msg); // TODO
	}
}

export default Form;
